# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The schedule commands module.

These commands are used in the following models:
SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - schedule.alarm[N].enable
    - schedule.alarm[N].fractionalseconds
    - schedule.alarm[N].period
    - schedule.alarm[N].ptpseconds
    - schedule.alarm[N].repetition
    - schedule.alarm[N].seconds
    - schedule.disable()
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING, Union

from ..helpers import (
    BaseTSPCmd,
    DefaultDictPassKeyToFactory,
    NoDeviceProvidedError,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class ScheduleAlarmItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``schedule.alarm[N]`` command tree.

    Info:
        - ``N``, the alarm number (1 or 2).

    Constants:
        - ``.EVENT_ID``: This constant describes the trigger event generated by the alarm N.

    Properties and methods:
        - ``.enable``: The ``schedule.alarm[N].enable`` attribute.
        - ``.fractionalseconds``: The ``schedule.alarm[N].fractionalseconds`` attribute.
        - ``.period``: The ``schedule.alarm[N].period`` attribute.
        - ``.ptpseconds``: The ``schedule.alarm[N].ptpseconds`` attribute.
        - ``.repetition``: The ``schedule.alarm[N].repetition`` attribute.
        - ``.seconds``: The ``schedule.alarm[N].seconds`` attribute.
    """

    EVENT_ID = "schedule.alarm[N].EVENT_ID"
    """str: This constant describes the trigger event generated by the alarm N."""

    def __init__(self, device: Optional["TSPDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        # pylint: disable=invalid-name
        self.EVENT_ID = self.EVENT_ID.replace(
            "[N]", f"[{self._cmd_syntax.rsplit('[', maxsplit=1)[-1].split(']', maxsplit=1)[0]}]"
        )

    @property
    def enable(self) -> str:
        """Access the ``schedule.alarm[N].enable`` attribute.

        Description:
            - This attribute enables or disables an alarm.

        Usage:
            - Accessing this property will send the ``print(schedule.alarm[N].enable)`` query.
            - Setting this property to a value will send the ``schedule.alarm[N].enable = value``
              command.

        TSP Syntax:
            ```
            - schedule.alarm[N].enable = value
            - print(schedule.alarm[N].enable)
            ```

        Info:
            - ``N``, the alarm number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".enable"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.enable)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @enable.setter
    def enable(self, value: Union[str, float]) -> None:
        """Access the ``schedule.alarm[N].enable`` attribute.

        Description:
            - This attribute enables or disables an alarm.

        Usage:
            - Accessing this property will send the ``print(schedule.alarm[N].enable)`` query.
            - Setting this property to a value will send the ``schedule.alarm[N].enable = value``
              command.

        TSP Syntax:
            ```
            - schedule.alarm[N].enable = value
            - print(schedule.alarm[N].enable)
            ```

        Info:
            - ``N``, the alarm number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".enable", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.enable = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def fractionalseconds(self) -> str:
        """Access the ``schedule.alarm[N].fractionalseconds`` attribute.

        Description:
            - This attribute describes the fractional seconds portion of the alarm time.

        Usage:
            - Accessing this property will send the ``print(schedule.alarm[N].fractionalseconds)``
              query.
            - Setting this property to a value will send the
              ``schedule.alarm[N].fractionalseconds = value`` command.

        TSP Syntax:
            ```
            - schedule.alarm[N].fractionalseconds = value
            - print(schedule.alarm[N].fractionalseconds)
            ```

        Info:
            - ``N``, the alarm number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".fractionalseconds"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.fractionalseconds)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.fractionalseconds`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @fractionalseconds.setter
    def fractionalseconds(self, value: Union[str, float]) -> None:
        """Access the ``schedule.alarm[N].fractionalseconds`` attribute.

        Description:
            - This attribute describes the fractional seconds portion of the alarm time.

        Usage:
            - Accessing this property will send the ``print(schedule.alarm[N].fractionalseconds)``
              query.
            - Setting this property to a value will send the
              ``schedule.alarm[N].fractionalseconds = value`` command.

        TSP Syntax:
            ```
            - schedule.alarm[N].fractionalseconds = value
            - print(schedule.alarm[N].fractionalseconds)
            ```

        Info:
            - ``N``, the alarm number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".fractionalseconds", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.fractionalseconds = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.fractionalseconds`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def period(self) -> str:
        """Access the ``schedule.alarm[N].period`` attribute.

        Description:
            - This attribute describes the time, in seconds, between adjacent firings of the alarm.

        Usage:
            - Accessing this property will send the ``print(schedule.alarm[N].period)`` query.
            - Setting this property to a value will send the ``schedule.alarm[N].period = value``
              command.

        TSP Syntax:
            ```
            - schedule.alarm[N].period = value
            - print(schedule.alarm[N].period)
            ```

        Info:
            - ``N``, the alarm number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".period"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.period)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.period`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @period.setter
    def period(self, value: Union[str, float]) -> None:
        """Access the ``schedule.alarm[N].period`` attribute.

        Description:
            - This attribute describes the time, in seconds, between adjacent firings of the alarm.

        Usage:
            - Accessing this property will send the ``print(schedule.alarm[N].period)`` query.
            - Setting this property to a value will send the ``schedule.alarm[N].period = value``
              command.

        TSP Syntax:
            ```
            - schedule.alarm[N].period = value
            - print(schedule.alarm[N].period)
            ```

        Info:
            - ``N``, the alarm number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".period", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.period = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.period`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def ptpseconds(self) -> str:
        """Access the ``schedule.alarm[N].ptpseconds`` attribute.

        Description:
            - The seconds portion of the alarm time in PTP seconds.

        Usage:
            - Accessing this property will send the ``print(schedule.alarm[N].ptpseconds)`` query.
            - Setting this property to a value will send the
              ``schedule.alarm[N].ptpseconds = value`` command.

        TSP Syntax:
            ```
            - schedule.alarm[N].ptpseconds = value
            - print(schedule.alarm[N].ptpseconds)
            ```

        Info:
            - ``N``, the alarm (1 or 2).
            - ``seconds``, the seconds portion of the alarm time in PTP seconds.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".ptpseconds"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.ptpseconds)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.ptpseconds`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @ptpseconds.setter
    def ptpseconds(self, value: Union[str, float]) -> None:
        """Access the ``schedule.alarm[N].ptpseconds`` attribute.

        Description:
            - The seconds portion of the alarm time in PTP seconds.

        Usage:
            - Accessing this property will send the ``print(schedule.alarm[N].ptpseconds)`` query.
            - Setting this property to a value will send the
              ``schedule.alarm[N].ptpseconds = value`` command.

        TSP Syntax:
            ```
            - schedule.alarm[N].ptpseconds = value
            - print(schedule.alarm[N].ptpseconds)
            ```

        Info:
            - ``N``, the alarm (1 or 2).
            - ``seconds``, the seconds portion of the alarm time in PTP seconds.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".ptpseconds", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.ptpseconds = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.ptpseconds`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def repetition(self) -> str:
        """Access the ``schedule.alarm[N].repetition`` attribute.

        Description:
            - This attribute describes the number of times an alarm repeats after the first alarm
              firing.

        Usage:
            - Accessing this property will send the ``print(schedule.alarm[N].repetition)`` query.
            - Setting this property to a value will send the
              ``schedule.alarm[N].repetition = value`` command.

        TSP Syntax:
            ```
            - schedule.alarm[N].repetition = value
            - print(schedule.alarm[N].repetition)
            ```

        Info:
            - ``N``, the alarm 1 or 2.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".repetition"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.repetition)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.repetition`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @repetition.setter
    def repetition(self, value: Union[str, float]) -> None:
        """Access the ``schedule.alarm[N].repetition`` attribute.

        Description:
            - This attribute describes the number of times an alarm repeats after the first alarm
              firing.

        Usage:
            - Accessing this property will send the ``print(schedule.alarm[N].repetition)`` query.
            - Setting this property to a value will send the
              ``schedule.alarm[N].repetition = value`` command.

        TSP Syntax:
            ```
            - schedule.alarm[N].repetition = value
            - print(schedule.alarm[N].repetition)
            ```

        Info:
            - ``N``, the alarm 1 or 2.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".repetition", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.repetition = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.repetition`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def seconds(self) -> str:
        """Access the ``schedule.alarm[N].seconds`` attribute.

        Description:
            - The seconds portion of the alarm time in UTC seconds.

        Usage:
            - Accessing this property will send the ``print(schedule.alarm[N].seconds)`` query.
            - Setting this property to a value will send the ``schedule.alarm[N].seconds = value``
              command.

        TSP Syntax:
            ```
            - schedule.alarm[N].seconds = value
            - print(schedule.alarm[N].seconds)
            ```

        Info:
            - ``N``, the alarm number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".seconds"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.seconds)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.seconds`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @seconds.setter
    def seconds(self, value: Union[str, float]) -> None:
        """Access the ``schedule.alarm[N].seconds`` attribute.

        Description:
            - The seconds portion of the alarm time in UTC seconds.

        Usage:
            - Accessing this property will send the ``print(schedule.alarm[N].seconds)`` query.
            - Setting this property to a value will send the ``schedule.alarm[N].seconds = value``
              command.

        TSP Syntax:
            ```
            - schedule.alarm[N].seconds = value
            - print(schedule.alarm[N].seconds)
            ```

        Info:
            - ``N``, the alarm number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".seconds", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.seconds = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.seconds`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Schedule(BaseTSPCmd):
    """The ``schedule`` command tree.

    Properties and methods:
        - ``.alarm``: The ``schedule.alarm[N]`` command tree.
        - ``.disable()``: The ``schedule.disable()`` function.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "schedule") -> None:
        super().__init__(device, cmd_syntax)
        self._alarm: Dict[int, ScheduleAlarmItem] = DefaultDictPassKeyToFactory(
            lambda x: ScheduleAlarmItem(device, f"{self._cmd_syntax}.alarm[{x}]")
        )

    @property
    def alarm(self) -> Dict[int, ScheduleAlarmItem]:
        """Return the ``schedule.alarm[N]`` command tree.

        Info:
            - ``N``, the alarm number (1 or 2).

        Constants:
            - ``.EVENT_ID``: This constant describes the trigger event generated by the alarm N.

        Sub-properties and sub-methods:
            - ``.enable``: The ``schedule.alarm[N].enable`` attribute.
            - ``.fractionalseconds``: The ``schedule.alarm[N].fractionalseconds`` attribute.
            - ``.period``: The ``schedule.alarm[N].period`` attribute.
            - ``.ptpseconds``: The ``schedule.alarm[N].ptpseconds`` attribute.
            - ``.repetition``: The ``schedule.alarm[N].repetition`` attribute.
            - ``.seconds``: The ``schedule.alarm[N].seconds`` attribute.
        """
        return self._alarm

    def disable(self) -> None:
        """Run the ``schedule.disable()`` function.

        Description:
            - This function disables all alarms.

        TSP Syntax:
            ```
            - schedule.disable()
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.disable()"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.disable()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The trigger commands module.

These commands are used in the following models:
AFG3K, AFG3KB, AFG3KC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - TRIGger:SEQuence:IMMediate
    - TRIGger:SEQuence:SLOPe {POSitive|NEGative}
    - TRIGger:SEQuence:SLOPe?
    - TRIGger:SEQuence:SOURce {TIMer|EXTernal}
    - TRIGger:SEQuence:SOURce?
    - TRIGger:SEQuence:TIMer <seconds>
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class TriggerSequenceTimer(SCPICmdWrite):
    """The ``TRIGger:SEQuence:TIMer`` command.

    Description:
        - This command sets or queries the period of an internal clock when you select the internal
          clock as the trigger source with the TRIGger[``:SEQuence``]``:SOURce`` command. The
          setting range is 1 µs to 500.0 s.

    Usage:
        - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:TIMer value`` command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:TIMer <seconds>
        ```

    Info:
        - ``<seconds>::=<NRf>[<units>]``
    """


class TriggerSequenceSource(SCPICmdWrite, SCPICmdRead):
    """The ``TRIGger:SEQuence:SOURce`` command.

    Description:
        - This command sets or queries the trigger source for an external trigger signal.

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger:SEQuence:SOURce?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:SOURce?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:SOURce value``
          command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:SOURce {TIMer|EXTernal}
        - TRIGger:SEQuence:SOURce?
        ```

    Info:
        - ``TIMer`` specifies an internal clock as the trigger source.
        - ``EXTernal`` specifies an external trigger input as the trigger source.
    """


class TriggerSequenceSlope(SCPICmdWrite, SCPICmdRead):
    """The ``TRIGger:SEQuence:SLOPe`` command.

    Description:
        - This command sets or queries the slope of trigger signal.

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger:SEQuence:SLOPe?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:SLOPe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:SLOPe value`` command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:SLOPe {POSitive|NEGative}
        - TRIGger:SEQuence:SLOPe?
        ```

    Info:
        - ``POSitive`` indicates that the event occurs on the rising edge of the external trigger
          signal.
        - ``NEGative`` indicates that the event occurs on the falling edge of the external trigger
          signal.
    """


class TriggerSequenceImmediate(SCPICmdWriteNoArguments):
    """The ``TRIGger:SEQuence:IMMediate`` command.

    Description:
        - This command forces a trigger event to occur.

    Usage:
        - Using the ``.write()`` method will send the ``TRIGger:SEQuence:IMMediate`` command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:IMMediate
        ```
    """


class TriggerSequence(SCPICmdRead):
    """The ``TRIGger:SEQuence`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger:SEQuence?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.slope``: The ``TRIGger:SEQuence:SLOPe`` command.
        - ``.source``: The ``TRIGger:SEQuence:SOURce`` command.
        - ``.timer``: The ``TRIGger:SEQuence:TIMer`` command.
        - ``.immediate``: The ``TRIGger:SEQuence:IMMediate`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._slope = TriggerSequenceSlope(device, f"{self._cmd_syntax}:SLOPe")
        self._source = TriggerSequenceSource(device, f"{self._cmd_syntax}:SOURce")
        self._timer = TriggerSequenceTimer(device, f"{self._cmd_syntax}:TIMer")
        self._immediate = TriggerSequenceImmediate(device, f"{self._cmd_syntax}:IMMediate")

    @property
    def slope(self) -> TriggerSequenceSlope:
        """Return the ``TRIGger:SEQuence:SLOPe`` command.

        Description:
            - This command sets or queries the slope of trigger signal.

        Usage:
            - Using the ``.query()`` method will send the ``TRIGger:SEQuence:SLOPe?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:SLOPe?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:SLOPe value``
              command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:SLOPe {POSitive|NEGative}
            - TRIGger:SEQuence:SLOPe?
            ```

        Info:
            - ``POSitive`` indicates that the event occurs on the rising edge of the external
              trigger signal.
            - ``NEGative`` indicates that the event occurs on the falling edge of the external
              trigger signal.
        """
        return self._slope

    @property
    def source(self) -> TriggerSequenceSource:
        """Return the ``TRIGger:SEQuence:SOURce`` command.

        Description:
            - This command sets or queries the trigger source for an external trigger signal.

        Usage:
            - Using the ``.query()`` method will send the ``TRIGger:SEQuence:SOURce?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:SOURce?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:SOURce value``
              command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:SOURce {TIMer|EXTernal}
            - TRIGger:SEQuence:SOURce?
            ```

        Info:
            - ``TIMer`` specifies an internal clock as the trigger source.
            - ``EXTernal`` specifies an external trigger input as the trigger source.
        """
        return self._source

    @property
    def timer(self) -> TriggerSequenceTimer:
        """Return the ``TRIGger:SEQuence:TIMer`` command.

        Description:
            - This command sets or queries the period of an internal clock when you select the
              internal clock as the trigger source with the TRIGger[``:SEQuence``]``:SOURce``
              command. The setting range is 1 µs to 500.0 s.

        Usage:
            - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:TIMer value``
              command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:TIMer <seconds>
            ```

        Info:
            - ``<seconds>::=<NRf>[<units>]``
        """
        return self._timer

    @property
    def immediate(self) -> TriggerSequenceImmediate:
        """Return the ``TRIGger:SEQuence:IMMediate`` command.

        Description:
            - This command forces a trigger event to occur.

        Usage:
            - Using the ``.write()`` method will send the ``TRIGger:SEQuence:IMMediate`` command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:IMMediate
            ```
        """
        return self._immediate


class Trigger(SCPICmdRead):
    """The ``TRIGger`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.sequence``: The ``TRIGger:SEQuence`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "TRIGger") -> None:
        super().__init__(device, cmd_syntax)
        self._sequence = TriggerSequence(device, f"{self._cmd_syntax}:SEQuence")

    @property
    def sequence(self) -> TriggerSequence:
        """Return the ``TRIGger:SEQuence`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``TRIGger:SEQuence?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.slope``: The ``TRIGger:SEQuence:SLOPe`` command.
            - ``.source``: The ``TRIGger:SEQuence:SOURce`` command.
            - ``.timer``: The ``TRIGger:SEQuence:TIMer`` command.
            - ``.immediate``: The ``TRIGger:SEQuence:IMMediate`` command.
        """
        return self._sequence



"""The clear commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CLEAR {ALL}
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Clear(SCPICmdWrite):
    """The ``CLEAR`` command.

    Description:
        - This command clears acquisitions, measurements, and waveforms.

    Usage:
        - Using the ``.write(value)`` method will send the ``CLEAR value`` command.

    SCPI Syntax:
        ```
        - CLEAR {ALL}
        ```
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "CLEAR") -> None:
        super().__init__(device, cmd_syntax)



"""The auxout commands module.

These commands are used in the following models:
MSO2

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - AUXout:EDGE {RISing|FALling}
    - AUXout:EDGE?
    - AUXout:SOUrce {ATRIGger|OFF}
    - AUXout:SOUrce?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class AuxoutSource(SCPICmdWrite, SCPICmdRead):
    """The ``AUXout:SOUrce`` command.

    Description:
        - This command sets or queries the source at the AFG/AUX OUT, shared BNC connection.

    Usage:
        - Using the ``.query()`` method will send the ``AUXout:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXout:SOUrce?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXout:SOUrce value`` command.

    SCPI Syntax:
        ```
        - AUXout:SOUrce {ATRIGger|OFF}
        - AUXout:SOUrce?
        ```

    Info:
        - ``ATRIGger`` sets the source at the AFG/AUX OUT, shared BNC connector to the main trigger
          and turns off the AFG output (if on).
        - ``OFF`` turns off the output of the AFG/AUX OUT, shared BNC connector, this will turn off
          the AFG (if on).
    """


class AuxoutEdge(SCPICmdWrite, SCPICmdRead):
    """The ``AUXout:EDGE`` command.

    Description:
        - This command sets or queries the direction in which the Auxiliary Output signal will
          transition when a trigger occurs.

    Usage:
        - Using the ``.query()`` method will send the ``AUXout:EDGE?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXout:EDGE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXout:EDGE value`` command.

    SCPI Syntax:
        ```
        - AUXout:EDGE {RISing|FALling}
        - AUXout:EDGE?
        ```

    Info:
        - ``RISing`` sets the polarity to the rising edge.
        - ``FALling`` sets the polarity to the falling edge.
    """


class Auxout(SCPICmdRead):
    """The ``AUXout`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AUXout?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXout?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.edge``: The ``AUXout:EDGE`` command.
        - ``.source``: The ``AUXout:SOUrce`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "AUXout") -> None:
        super().__init__(device, cmd_syntax)
        self._edge = AuxoutEdge(device, f"{self._cmd_syntax}:EDGE")
        self._source = AuxoutSource(device, f"{self._cmd_syntax}:SOUrce")

    @property
    def edge(self) -> AuxoutEdge:
        """Return the ``AUXout:EDGE`` command.

        Description:
            - This command sets or queries the direction in which the Auxiliary Output signal will
              transition when a trigger occurs.

        Usage:
            - Using the ``.query()`` method will send the ``AUXout:EDGE?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXout:EDGE?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXout:EDGE value`` command.

        SCPI Syntax:
            ```
            - AUXout:EDGE {RISing|FALling}
            - AUXout:EDGE?
            ```

        Info:
            - ``RISing`` sets the polarity to the rising edge.
            - ``FALling`` sets the polarity to the falling edge.
        """
        return self._edge

    @property
    def source(self) -> AuxoutSource:
        """Return the ``AUXout:SOUrce`` command.

        Description:
            - This command sets or queries the source at the AFG/AUX OUT, shared BNC connection.

        Usage:
            - Using the ``.query()`` method will send the ``AUXout:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXout:SOUrce?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXout:SOUrce value`` command.

        SCPI Syntax:
            ```
            - AUXout:SOUrce {ATRIGger|OFF}
            - AUXout:SOUrce?
            ```

        Info:
            - ``ATRIGger`` sets the source at the AFG/AUX OUT, shared BNC connector to the main
              trigger and turns off the AFG output (if on).
            - ``OFF`` turns off the output of the AFG/AUX OUT, shared BNC connector, this will turn
              off the AFG (if on).
        """
        return self._source



# pylint: disable=line-too-long
"""The fpanel commands module.

These commands are used in the following models:
MSO2

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - FPAnel:PRESS {AUTOset| BUS| CH1<x>| CLEAR| DEFaultsetup| FORCetrig| GPKNOB1| GPKNOB2| HORZPOS| MATh| REF| RUNSTop| SETTO50| SINGleseq| TOUCHSCReen| TRIGMode| USER| VERTPOS| VERTSCALE}
    - FPAnel:TURN {GPKNOB1| GPKNOB2| HORZPOS| HORZScale| TRIGLevel| VERTPOS| VERTSCALE} [,<NR1>]
    ```
"""  # noqa: E501

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class FpanelTurn(SCPICmdWrite):
    """The ``FPAnel:TURN`` command.

    Description:
        - This command is used to emulate a knob turn. The optional NR1 specifies the number of
          clicks where negative values indicate counter clockwise. If not specified, the default of
          1 click is used indicating the knob is turned clockwise 1 click.

    Usage:
        - Using the ``.write(value)`` method will send the ``FPAnel:TURN value`` command.

    SCPI Syntax:
        ```
        - FPAnel:TURN {GPKNOB1| GPKNOB2| HORZPOS| HORZScale| TRIGLevel| VERTPOS| VERTSCALE} [,<NR1>]
        ```

    Info:
        - ``<NR1>`` is the number of clicks to turn the knob.
    """


class FpanelPress(SCPICmdWrite):
    """The ``FPAnel:PRESS`` command.

    Description:
        - This command is used to emulate a button press. When used with knob enumerations, this
          command pushes the knob. Use the ``FPANEL:TURN`` command to emulate knob turns.

    Usage:
        - Using the ``.write(value)`` method will send the ``FPAnel:PRESS value`` command.

    SCPI Syntax:
        ```
        - FPAnel:PRESS {AUTOset| BUS| CH1<x>| CLEAR| DEFaultsetup| FORCetrig| GPKNOB1| GPKNOB2| HORZPOS| MATh| REF| RUNSTop| SETTO50| SINGleseq| TOUCHSCReen| TRIGMode| USER| VERTPOS| VERTSCALE}
        ```

    Info:
        - ``AUTOset`` Arguments are the following instrument buttons.
        - ``BUS`` Arguments are the following instrument buttons.
        - ``CH1<x>`` Arguments are the following instrument buttons.
        - ``CLEAR`` Arguments are the following instrument buttons.
        - ``DEFaultsetup`` Arguments are the following instrument buttons.
        - ``FORCetrig`` Arguments are the following instrument buttons.
        - ``GPKNOB1`` Arguments are the following instrument buttons.
        - ``GPKNOB2`` Arguments are the following instrument buttons.
        - ``HORZPOS`` Arguments are the following instrument buttons.
        - ``MATh`` Arguments are the following instrument buttons.
        - ``REF`` Arguments are the following instrument buttons.
        - ``RUNSTop`` Arguments are the following instrument buttons.
        - ``SETTO50`` Arguments are the following instrument buttons.
        - ``SINGleseq`` Arguments are the following instrument buttons.
        - ``TOUCHSCReen`` Arguments are the following instrument buttons.
        - ``TRIGMode`` Arguments are the following instrument buttons.
        - ``USER`` Arguments are the following instrument buttons.
        - ``VERTPOS`` Arguments are the following instrument buttons.
        - ``VERTSCALE`` Arguments are the following instrument buttons.
    """  # noqa: E501


class Fpanel(SCPICmdRead):
    """The ``FPAnel`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``FPAnel?`` query.
        - Using the ``.verify(value)`` method will send the ``FPAnel?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.press``: The ``FPAnel:PRESS`` command.
        - ``.turn``: The ``FPAnel:TURN`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "FPAnel") -> None:
        super().__init__(device, cmd_syntax)
        self._press = FpanelPress(device, f"{self._cmd_syntax}:PRESS")
        self._turn = FpanelTurn(device, f"{self._cmd_syntax}:TURN")

    @property
    def press(self) -> FpanelPress:
        """Return the ``FPAnel:PRESS`` command.

        Description:
            - This command is used to emulate a button press. When used with knob enumerations, this
              command pushes the knob. Use the ``FPANEL:TURN`` command to emulate knob turns.

        Usage:
            - Using the ``.write(value)`` method will send the ``FPAnel:PRESS value`` command.

        SCPI Syntax:
            ```
            - FPAnel:PRESS {AUTOset| BUS| CH1<x>| CLEAR| DEFaultsetup| FORCetrig| GPKNOB1| GPKNOB2| HORZPOS| MATh| REF| RUNSTop| SETTO50| SINGleseq| TOUCHSCReen| TRIGMode| USER| VERTPOS| VERTSCALE}
            ```

        Info:
            - ``AUTOset`` Arguments are the following instrument buttons.
            - ``BUS`` Arguments are the following instrument buttons.
            - ``CH1<x>`` Arguments are the following instrument buttons.
            - ``CLEAR`` Arguments are the following instrument buttons.
            - ``DEFaultsetup`` Arguments are the following instrument buttons.
            - ``FORCetrig`` Arguments are the following instrument buttons.
            - ``GPKNOB1`` Arguments are the following instrument buttons.
            - ``GPKNOB2`` Arguments are the following instrument buttons.
            - ``HORZPOS`` Arguments are the following instrument buttons.
            - ``MATh`` Arguments are the following instrument buttons.
            - ``REF`` Arguments are the following instrument buttons.
            - ``RUNSTop`` Arguments are the following instrument buttons.
            - ``SETTO50`` Arguments are the following instrument buttons.
            - ``SINGleseq`` Arguments are the following instrument buttons.
            - ``TOUCHSCReen`` Arguments are the following instrument buttons.
            - ``TRIGMode`` Arguments are the following instrument buttons.
            - ``USER`` Arguments are the following instrument buttons.
            - ``VERTPOS`` Arguments are the following instrument buttons.
            - ``VERTSCALE`` Arguments are the following instrument buttons.
        """  # noqa: E501
        return self._press

    @property
    def turn(self) -> FpanelTurn:
        """Return the ``FPAnel:TURN`` command.

        Description:
            - This command is used to emulate a knob turn. The optional NR1 specifies the number of
              clicks where negative values indicate counter clockwise. If not specified, the default
              of 1 click is used indicating the knob is turned clockwise 1 click.

        Usage:
            - Using the ``.write(value)`` method will send the ``FPAnel:TURN value`` command.

        SCPI Syntax:
            ```
            - FPAnel:TURN {GPKNOB1| GPKNOB2| HORZPOS| HORZScale| TRIGLevel| VERTPOS| VERTSCALE} [,<NR1>]
            ```

        Info:
            - ``<NR1>`` is the number of clicks to turn the knob.
        """  # noqa: E501
        return self._turn



"""Helper objects for command drivers.

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.
"""

from .generic_commands import (
    DefaultDictPassKeyToFactory,
    NoDeviceProvidedError,
    ValidatedDynamicNumberCmd,
)
from .scpi_commands import (
    BaseSCPICmd,
    DefaultDictDeviceCommunication,
    SCPICmdRead,
    SCPICmdReadWithArguments,
    SCPICmdWrite,
    SCPICmdWriteNoArguments,
    ValidatedChannel,
    ValidatedDigitalBit,
)
from .tsp_commands import BaseTSPCmd

__all__ = [
    "BaseSCPICmd",
    "BaseTSPCmd",
    "DefaultDictDeviceCommunication",
    "DefaultDictPassKeyToFactory",
    "NoDeviceProvidedError",
    "SCPICmdRead",
    "SCPICmdReadWithArguments",
    "SCPICmdWrite",
    "SCPICmdWriteNoArguments",
    "ValidatedChannel",
    "ValidatedDigitalBit",
    "ValidatedDynamicNumberCmd",
]



"""The curvenext commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CURVENext?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Curvenext(SCPICmdRead):
    """The ``CURVENext`` command.

    Description:
        - This query-only command returns unique waveform data from the instrument. This query
          performs just like CURVE?, except multiple uses guarantee that the waveform returned is
          always a new acquisition since the previous CURVENEXT. Note that if the instrument is
          acquiring waveform records at a slow rate (high resolution mode), you must configure the
          controller for long timeout thresholds. Data will not be transferred until a new waveform
          is acquired since the previous ``:CURVENext?`` response.

    Usage:
        - Using the ``.query()`` method will send the ``CURVENext?`` query.
        - Using the ``.verify(value)`` method will send the ``CURVENext?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CURVENext?
        ```
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "CURVENext") -> None:
        super().__init__(device, cmd_syntax)
